@@ -1,7 +1,9 @@
 "panel.interaction2wt" <-
 function(x, y, subscripts, 
-                                 responselab, trace.values, factor.levels,
-                                 fun=mean, ...) {
+         responselab, trace.values, factor.levels,
+         fun=mean, ...,
+         key.in=NULL ## list of key arguments
+         ) {
   tpg <- trellis.par.get("superpose.line")
   if.R(r={
     ## This loop is needed because
@@ -22,18 +24,19 @@ function(x, y, subscripts,
   },
        s={})
   if (column==1) {
-    row.label <- names(factor.levels)[row]
-    row.levels <- factor.levels[[row]]$levels
-    n.levels <- min(length(tpg$col),length(row.levels))
-    key.list <- list(title=row.label ,
+
+    trace.factor <- dimnames(factor.levels)[[1]][row]
+    trace.levels <- factor.levels[[trace.factor, "levels"]]
+    n.levels <- min(length(tpg$col), length(trace.levels))
+    key.list <- list(title=trace.factor,
                      cex.title=1,
                      corner=c(.5,.5), border=TRUE,
-#                     x=-x.center*1.3, y=y.center,
-                     text=list(text=row.levels, cex=.8),
+                     text=list(text=trace.levels, cex=.8),
                      lines=list(
                        col=tpg$col[1:n.levels],
                        lty=tpg$lty[1:n.levels],
                        lwd=tpg$lwd[1:n.levels]))
+    key.list[names(key.in)] <- key.in
     if.R(r=draw.key(key.list, draw=TRUE),
          s=do.call(key, key.list))
     return()
@@ -43,91 +46,50 @@ function(x, y, subscripts,
 
 
 
-  cell <- if.R(r=get("panel.number", pos=sf2),
+  cell <- if.R(r=panel.number(),
                s=get("cell", frame=sys.parent()))
   which.cell <- if.R(r=as.vector(
                        matrix(seq(get("plots.per.page", pos=sf2)),
-                              nrow=get("cols.per.page", pos=sf2))
+                              nrow=cols.per.page)
                               [-1,]),
                      s=get("which.cell", frame=sys.parent()))
   this.cell <- match(cell, which.cell)
   these.labels <- 
-    if.R(r=c(trace.factor=names(factor.levels)[get("row", pos=sf2)],
-           x.factor=names(factor.levels)[get("column", pos=sf2)-1]),
-         s=get("panel.labels", frame=sys.parent())[this.cell,])
-  fac.levels <- factor.levels[[these.labels["trace.factor"]]]$levels
-  x.levels <- factor.levels[[these.labels["x.factor"]]]
-##browser()
-  if (x.levels$class[[1]] == "ordered") {
-     old.warn <- options(warn=-1)
-     if (!any(is.na(as.numeric(x.levels$levels))))
-       x.levels <- as.numeric(x.levels$levels)
-     else x.levels <-  x.levels$levels
-     options(old.warn)
-   }
-   else
-    x.levels <- x.levels$levels
+    if.R(r=
+         c(trace.factor=dimnames(factor.levels)[[1]][row],
+           x.factor=dimnames(factor.levels)[[1]][column-1])
+         ,s=
+         get("panel.labels", frame=sys.parent())[this.cell,])
+  
+  trace.levels <- factor.levels[[these.labels["trace.factor"], "levels"]]
+  x.levels <- factor.levels[[these.labels["x.factor"], "position"]]  ## change of name ok
+  
   if (these.labels["x.factor"] == these.labels["trace.factor"]) {
     box.par <- list(box.dot=trellis.par.get("box.dot"),
                     box.rectangle=trellis.par.get("box.rectangle"),
                     box.umbrella=trellis.par.get("box.umbrella"))
     box.col <- lapply(box.par, function(x) list(col=x$col))
    if.R(r={
-     tpg.col <- rep(tpg$col, length=length(fac.levels))
-     tpg.lty <- rep(tpg$lty, length=length(fac.levels))
-     for (i in 1:length(fac.levels)) {
+     tpg.col <- rep(tpg$col, length=length(trace.levels))
+     tpg.lty <- rep(tpg$lty, length=length(trace.levels))
+     for (i in 1:length(trace.levels)) {
        for (j in seq(along=box.col))
          box.col[[j]]$col <- tpg.col[i]
        box.col$box.rectangle$lty <- tpg.lty[i]
        trellis.par.set(box.col)
-#        panel.bwplot(x[x==x.levels[i]], y[x==x.levels[i]], horizontal=FALSE)
-#        panel.bwplot(i, y[x==x.levels[i]], horizontal=FALSE)
-       position <- factor.levels[[these.labels["x.factor"]]]$position
-       if (!is.null(position)) {
-         o.p <- (x==position[i])
-         yx <- y[o.p]
-         ix <-  rep(position[i], length(yx))
-         panel.bwplot.hh(ix, yx, horizontal = FALSE,
-                         at=position[i])
-       }
-       else {
-         if (is.numeric(x.levels)) {
-           yx <- y[x == x.levels[i]]
-           ix <- rep(i, length(yx))
-           panel.bwplot.hh(ix, yx, horizontal = FALSE,
-                           at=as.numeric(x.levels[i]))
-         }
-         else {
-           yx <- y[x.levels[x] == x.levels[i]]
-           ix <- rep(i, length(yx))
-           panel.bwplot.hh(ix, yx, horizontal = FALSE,
-                           at=i)
-         }
-       }
-## browser()        
+       
+       position <- factor.levels[[these.labels["x.factor"], "position"]]
+       x.position <- as.position(x)
+       yx <- y[x.position == position[i]]
+       ix <- rep(position[i], length(yx))
+       panel.bwplot(ix, yx, horizontal = FALSE)
      }
      trellis.par.set(lapply(box.par, function(x) list(col=x$col)))
    }
-        ##       panel.bwplot(x, y, horizontal=FALSE, col=tpg$col[1])
-        ,
-         s={
-    position <- factor.levels[[these.labels["x.factor"]]]$position
-    if (!is.null(position)) {
-      ## browser()
-      o.p <- order(position[match(x, position)])
-      panel.bwplot.intermediate.hh((position[match(x,position)])[o.p], y[o.p],
-                                   transpose=TRUE,
-                                   factor.levels=factor.levels, ...)
-    }
-    else {
-      if (is.numeric(x.levels))
-        panel.bwplot.intermediate.hh(as.numeric(x), y, transpose=TRUE,
-                                   factor.levels=factor.levels, ...)
-      else
-        panel.bwplot.intermediate.hh(x, y, transpose=TRUE,
-                                   factor.levels=factor.levels, ...)
-    }
-})
+        ,s=
+        panel.bwplot.intermediate.hh(as.position(x), y, transpose=TRUE,
+                                     at=position(x), ...)
+        )
   }
   else {
     tab <- tapply(y, list(x, trace.values[subscripts]), fun)
@@ -137,12 +99,7 @@ function(x, y, subscripts,
     tpg2.lty <- rep(tpg2$lty, length=ncol(tab))
     if.R(r=for (j in 1:ncol(tab))
          llines(x=su.x, y=tab[,j], col=tpg2.col[j], lty=tpg2.lty[j]),
-         ##panel.linejoin(x, y, horizontal=FALSE),
          s=
-    ## ## from interaction.plot:
-    ## tab <-
-    ##   tapply(response,
-    ##          list(unclass(x.factor), unclass(trace.factor)), fun)
     matlines(su.x, tab, col=tpg$col, lty=tpg$lty))
   }
 
